from numpy import *
from math import *

from OpenGL.GL import *
from OpenGL.GLU import *

from constants import *

class Disk:
	def __init__(self, pos, world):
		self.world = world
	
		self.pos = array(pos, "f")
		self.rotation = 0
		
		self.speed = 0
		self.angularV = 0
		
		self.bFinished = False
		
		self.drawDList = glGenLists(1)
		
		glNewList(self.drawDList, GL_COMPILE)
		for j in range(4):
			if j % 2:
				glColor4f(*DISK_COLOR_1)
			else:
				glColor4f(*DISK_COLOR_2)		
		
			glBegin(GL_TRIANGLE_STRIP)	
			for i in range(46):
				angle = (i + (j * 45)) * pi / 90 
				
				glVertex2f(0.0, 0.0)
				glVertex2f(DISK_RADIUS * cos(angle), DISK_RADIUS * sin(angle))
		
			
			glEnd()
		glEndList()		
		
	def getRotation(self):
		return self.rotation
		
	def isFinished(self):
		return self.bFinished
		
	def update(self):
		self.speed *= 0.9
		self.angularV *= 0.99
	
		self.speed += self.world.getVectorAtPoint(self.pos) / DISK_MASS
		
		self.angularV += self.world.getVectorAtPoint(array((self.pos[0] + DISK_RADIUS, self.pos[1])))[1] / DISK_MOI
		self.angularV -= self.world.getVectorAtPoint(array((self.pos[0] - DISK_RADIUS, self.pos[1])))[1] / DISK_MOI
		self.angularV -= self.world.getVectorAtPoint(array((self.pos[0], self.pos[1] + DISK_RADIUS)))[0] / DISK_MOI
		self.angularV += self.world.getVectorAtPoint(array((self.pos[0], self.pos[1] - DISK_RADIUS)))[0] / DISK_MOI
			
		self.pos += self.speed
		self.rotation += self.angularV

		# Check for out of bounds
		if self.pos[0] <= BORDER_WIDTH + DISK_RADIUS:
			# Simple reflection of x speed
			self.speed[0] = -self.speed[0] * COLLISION_LOSS
			
			tangSpeed = self.speed[1]
			angVel = self.angularV

			self.speed[1] -= (tangSpeed - (DISK_RADIUS * angVel)) / 32
			self.angularV += (tangSpeed - (DISK_RADIUS * angVel)) / 32
			
			self.pos[0] = BORDER_WIDTH + DISK_RADIUS
			
		elif self.pos[0] >= SCREEN_RES[0] - BORDER_WIDTH - DISK_RADIUS:
			self.speed[0] = -self.speed[0] * COLLISION_LOSS
			
			tangSpeed = self.speed[1]
			angVel = self.angularV
			
			self.speed[1] += (tangSpeed + (DISK_RADIUS * angVel)) / 32
			self.angularV -= (tangSpeed + (DISK_RADIUS * angVel)) / 32
			
			self.pos[0] = SCREEN_RES[0] - BORDER_WIDTH - DISK_RADIUS
				
		if self.pos[1] <= BORDER_WIDTH + DISK_RADIUS:
			self.speed[1] = -self.speed[1] * COLLISION_LOSS
			
			tangSpeed = self.speed[0]
			angVel = self.angularV
			
			self.speed[0] += (tangSpeed + (DISK_RADIUS * angVel)) / 32
			self.angularV -= (tangSpeed + (DISK_RADIUS * angVel)) / 32
			
			self.pos[1] = BORDER_WIDTH + DISK_RADIUS
				
		elif self.pos[1] >= SCREEN_RES[1] - BORDER_WIDTH - DISK_RADIUS:
			self.speed[1] = -self.speed[1] * COLLISION_LOSS
			
			tangSpeed = self.speed[0]
			angVel = self.angularV
			
			self.speed[0] -= (tangSpeed - (DISK_RADIUS * angVel)) / 32
			self.angularV += (tangSpeed - (DISK_RADIUS * angVel)) / 32	
			
			self.pos[1] = SCREEN_RES[1] - BORDER_WIDTH - DISK_RADIUS
			
		for wall in self.world.getWallList():
			# Find closest point to centre of circle
			# Check against radius
			# Find vector of centre -> point
			
			if self.pos[0] < wall.left:
				if self.pos[1] < wall.bottom:
					# Possible bottom left impact. Must be with corner.
					collisionVector = self.pos - array(wall.bottomleft, "f")
					collisionVectorMagSq = sum(collisionVector ** 2)
					if collisionVectorMagSq < DISK_RADIUS_SQ:
						# Collision!
						self.pos = array(wall.bottomleft, "f") + DISK_RADIUS_SQ * collisionVector / collisionVectorMagSq
						
						# See http://planetmath.org/encyclopedia/DerivationOf2DReflectionMatrix.html
						"""xy2 = 2 * sum(collisionVector)
						uxSq = collisionVector[0] ** 2
						uySq = collisionVector[1] ** 2
						reflectionMatrix = (1.0 / collisionVectorMagSq) * matrix(((uxSq - uySq, xy2), (xy2, uySq - uxSq)))
						
						tangSpeed = 1 - ((1.0 / collisionVectorMagSq) * dot(collisionVector, self.speed))
						angVel = self.angularV
						
						self.speed += (tangSpeed + (DISK_RADIUS * angVel)) / 32
						self.angularV -= (sum(tangSpeed) + (DISK_RADIUS * angVel)) / 32"""
				
				elif self.pos[1] > wall.top:
					# Possible top left impact. Must be with corner.
					collisionVector = self.pos - array(wall.topleft, "f")
					collisionVectorMagSq = sum(collisionVector ** 2)
					if collisionVectorMagSq < DISK_RADIUS_SQ:
						self.pos = array(wall.topleft, "f") + DISK_RADIUS_SQ * collisionVector / collisionVectorMagSq
						
				# Direct impact on LHS
				elif self.pos[0] + DISK_RADIUS > wall.left and self.pos[0] < wall.centerx:
					print "Left direct"
					self.speed[0] = -self.speed[0] * COLLISION_LOSS
					
					tangSpeed = self.speed[1]
					angVel = self.angularV
					
					self.speed[1] += (tangSpeed + (DISK_RADIUS * angVel)) / 32
					self.angularV -= (tangSpeed + (DISK_RADIUS * angVel)) / 32
					
					self.pos[0] = wall.left - DISK_RADIUS
				
				
			elif self.pos[0] > wall.right:			
				if self.pos[1] < wall.bottom:
					# Possible bottom right impact. Must be with corner.
					collisionVector = self.pos - array(wall.bottomright, "f")
					collisionVectorMagSq = sum(collisionVector ** 2)
					if collisionVectorMagSq < DISK_RADIUS_SQ:
						self.pos = array(wall.bottomright, "f") + DISK_RADIUS_SQ * collisionVector / collisionVectorMagSq
				
				elif self.pos[1] > wall.top:
					# Possible top right impact. Must be with corner.
					collisionVector = self.pos - array(wall.topright, "f")
					collisionVectorMagSq = sum(collisionVector ** 2)
					if collisionVectorMagSq < DISK_RADIUS_SQ:
						self.pos = array(wall.topright, "f") + DISK_RADIUS_SQ * collisionVector / collisionVectorMagSq
						
				# Direct impact on RHS
				elif self.pos[0] - DISK_RADIUS < wall.right and self.pos[0] > wall.centerx:
					print "Right direct"
					self.speed[0] = -self.speed[0] * COLLISION_LOSS
					
					tangSpeed = self.speed[1]
					angVel = self.angularV

					self.speed[1] -= (tangSpeed - (DISK_RADIUS * angVel)) / 32
					self.angularV += (tangSpeed - (DISK_RADIUS * angVel)) / 32
					
					self.pos[0] = wall.right + DISK_RADIUS
			
			
			
			# Easy case - must be direct top or bottom impact
			else:
				if self.pos[1] + DISK_RADIUS > wall.bottom and self.pos[1] < wall.centery:

					self.speed[1] = -self.speed[1] * COLLISION_LOSS
					
					tangSpeed = self.speed[0]
					angVel = self.angularV
					
					self.speed[0] -= (tangSpeed - (DISK_RADIUS * angVel)) / 32
					self.angularV += (tangSpeed - (DISK_RADIUS * angVel)) / 32	
					
					self.pos[1] = wall.bottom - DISK_RADIUS			
					
				elif self.pos[1] - DISK_RADIUS < wall.top and self.pos[1] > wall.centery:
					print "Top direct"
					self.speed[1] = -self.speed[1] * COLLISION_LOSS
					
					tangSpeed = self.speed[0]
					angVel = self.angularV
					
					self.speed[0] += (tangSpeed + (DISK_RADIUS * angVel)) / 32
					self.angularV -= (tangSpeed + (DISK_RADIUS * angVel)) / 32				
				
					self.pos[1] = wall.top + DISK_RADIUS
			
		if self.world.checkVictoryForDisk(self.pos):
			self.bFinished = True

			
		
	def draw(self):
		glPushMatrix()
		glTranslatef(self.pos[0], self.pos[1], 0.0)
		glRotatef(self.getRotation() * RAD_TO_DEG, 0.0, 0.0, 1.0)
	
		glCallList(self.drawDList)
		
		glPopMatrix()
		
		
		
		