from math import *

from numpy import *
import pygame

from constants import *
from vector import *

def roundToGap(pos):
	for i in range(2):
		offset = pos[i] % (VECTOR_ARROW_SPACING / 2)
		if offset > VECTOR_ARROW_SPACING / 4:
			pos[i] -= VECTOR_ARROW_SPACING / 2 - offset
		elif offset != 0:
			pos[i] -= offset
			
	if pos[0] % VECTOR_ARROW_SPACING != pos[1] % VECTOR_ARROW_SPACING:
		pos[1] += VECTOR_ARROW_SPACING / 2
			
	return pos
	
def round1DToGap(pos, bIntermediate):
	if bIntermediate:
		offset = pos % (VECTOR_ARROW_SPACING / 2)
		if offset > VECTOR_ARROW_SPACING / 4:
			pos -= VECTOR_ARROW_SPACING / 2 - offset
		elif offset != 0:
			pos -= offset	
	else:
		offset = pos % (VECTOR_ARROW_SPACING) + VECTOR_ARROW_SPACING / 2
		if offset > VECTOR_ARROW_SPACING / 2:
			pos -= VECTOR_ARROW_SPACING - offset
		elif offset != 0:
			pos -= offset			
	
	return pos

class Object:
	def __init__(self, strength, orientation, bFixed):
		self.strength = strength
		self.orientation = orientation
		self.bFixed = bFixed
		
class RSquaredObject(Object):
	def __init__(self, pos, strength, orinetation, bFixed):
		Object.__init__(self, strength, orinetation, bFixed)
		self.pos = roundToGap(array(pos, "f"))

	def getEffectOnPoint(self, point, bFixed):
		if self.bFixed == bFixed:
			point = array(point, "f")
			relVec = rotateVector(self.pos - point, self.orientation)
			
			distanceSq = float(sum(relVec ** 2))
			
			if distanceSq:
				return relVec * self.strength / distanceSq
			else:
				return zeros(2, "f")		
		else:
			return array((0, 0), "f")
			
class GravityWell(RSquaredObject):
	def __init__(self, pos, strength, bFixed):
		RSquaredObject.__init__(self, pos, strength, 0, bFixed)
		
class Repeller(RSquaredObject):
	def __init__(self, pos, strength, bFixed):
		RSquaredObject.__init__(self, pos, strength, pi, bFixed)		
		
class Spinner(RSquaredObject):
	def __init__(self, pos, strength, bClockwise, bFixed):
		if bClockwise:
			angle = pi / 2
		else:
			angle = 3 * pi / 2
	
		RSquaredObject.__init__(self, pos, strength, angle, bFixed)		
		
class AreaObject(Object):
	def __init__(self, strength, orientation, areaRect, bFixed):
		Object.__init__(self, strength, orientation, bFixed)
		self.areaRect = areaRect
		
	def getEffectOnPoint(self, point, bFixed):
		if self.bFixed == bFixed:
			if self.areaRect.collidepoint(point):
				return rotateVector(array((1, 0), "f"), self.orientation) * self.strength
		else:
			return array((0, 0), "f")
			
class DefaultGravity(AreaObject):
	def __init__(self):
		AreaObject.__init__(self, 0.3, -pi / 2, pygame.Rect(0, 0, SCREEN_RES[0], SCREEN_RES[1]), True)
	
			
class HWall(pygame.Rect):
	def __init__(self, startX, endX, y):
		startX = round1DToGap(startX, True)
		endX = round1DToGap(endX, True)
		y = round1DToGap(y, False)
		
		pygame.Rect.__init__(self, startX, y + WALL_WIDTH / 2, endX - startX, -WALL_WIDTH)

class VWall(pygame.Rect):
	def __init__(self, x, startY, endY):
		x = round1DToGap(x, False)
		startY = round1DToGap(startY, True)
		endY = round1DToGap(endY, True)		
		
		pygame.Rect.__init__(self, x - WALL_WIDTH / 2, endY, WALL_WIDTH, startY - endY)		
	