from numpy import *
from math import *

import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *

from constants import *
from arrow import *
from disk import *
from levelData import *


class World:
	def __init__(self, iLevel):
		objList = getObjectListForLevel(iLevel)
		
		self.diskList = [Disk(getDiskPosForWorld(iLevel, i), self) for i in range(getNumDisksForWorld(iLevel))]
		self.arrowList = [[calculateArrow(((i + 1) * VECTOR_ARROW_SPACING, (j + 1) * VECTOR_ARROW_SPACING), objList) for j in range(getWorldY(iLevel))] for i in range(getWorldX(iLevel))]
		self.wallList = getWallListForLevel(iLevel)
		self.winPos = getWinPosForLevel(iLevel)
		
		self.bFinished = False
		
		self.iLevel = iLevel
		self.x = getWorldX(iLevel)
		self.y = getWorldY(iLevel)
		
		self.arrowDList = glGenLists(1)
		
		glNewList(self.arrowDList, GL_COMPILE)
		glBegin(GL_LINE_STRIP)
		for i in range(len(ARRROW_COORDS)):
			glVertex2f(ARRROW_COORDS[i][0], ARRROW_COORDS[i][1])
		glEnd()
		glEndList()
		
		self.borderDList = glGenLists(1)
		glNewList(self.borderDList, GL_COMPILE)
		glColor4f(*BORDER_COLOR)
		glBegin(GL_QUADS)
		# Bottom
		glVertex2f(0.0, 0.0)
		glVertex2f(SCREEN_RES[0], 0.0)
		glVertex2f(SCREEN_RES[0], BORDER_WIDTH)
		glVertex2f(0.0, BORDER_WIDTH)
		
		# Top
		glVertex2f(0.0, SCREEN_RES[1])
		glVertex2f(SCREEN_RES[0], SCREEN_RES[1])
		glVertex2f(SCREEN_RES[0], SCREEN_RES[1] - BORDER_WIDTH)
		glVertex2f(0.0, SCREEN_RES[1] - BORDER_WIDTH)
		
		# Left
		glVertex2f(0.0, 0.0)
		glVertex2f(BORDER_WIDTH, 0.0)
		glVertex2f(BORDER_WIDTH, SCREEN_RES[1])
		glVertex2f(0.0, SCREEN_RES[1])
		
		# Right
		glVertex2f(SCREEN_RES[0], 0.0)
		glVertex2f(SCREEN_RES[0] - BORDER_WIDTH, 0.0)
		glVertex2f(SCREEN_RES[0] - BORDER_WIDTH, SCREEN_RES[1])
		glVertex2f(SCREEN_RES[0], SCREEN_RES[1])
		glEnd()
		glEndList()	


		self.endTargetDList = glGenLists(1)
		
		glNewList(self.endTargetDList, GL_COMPILE)
		for j in range(4):
			if j % 2:
				glColor4f(*END_COLOR_1)
			else:
				glColor4f(*END_COLOR_2)		
		
			glBegin(GL_TRIANGLE_STRIP)	
			for i in range(46):
				angle = (i + (j * 45)) * pi / 90 
				
				glVertex2f(0.0, 0.0)
				glVertex2f(END_RADIUS * cos(angle), END_RADIUS * sin(angle))
		
			
			glEnd()
		glEndList()				
		
		
	def drawVectorFeild(self):
		glColor3f(1.0, 0.0, 0.0)
		glPushMatrix()
		
		for arrowSubList in self.arrowList:
			for arrow in arrowSubList:
				pos = arrow.getPos()
				vec = arrow.getVec()
				strength = arrow.getStrength()
				
				# Stick this in arrow display list
				fixedProp = arrow.getFixedProportion()
				glColor3f(1.0 - 2 * fixedProp, 0.0, fixedProp)
				
				glPushMatrix()
				if strength > 0:
					normVec = vec / strength
				
					glLineWidth(strength)
					
					# These are very slow. Make lots of lists, and remove translation and rotation from inside the loop
					glTranslatef(pos[0], pos[1], 0.0)
					glRotatef(atan2(normVec[1], normVec[0]) * RAD_TO_DEG, 0.0, 0.0, 1.0)
				
					glCallList(self.arrowDList)
					
				else:
					glTranslatef(pos[0], pos[1], 0.0)
					
					glBegin(GL_POINTS)
					glVertex2f(0.0, 0.0)
					glEnd()
					
				glPopMatrix()
					
		glPopMatrix()
		
	def drawBorder(self):
		glCallList(self.borderDList)
		
	def drawWalls(self):
		glColor4f(*WALL_COLOUR)
		for wall in self.wallList:
			glBegin(GL_QUADS)
			glVertex2f(wall.left, wall.top)
			glVertex2f(wall.right, wall.top)
			glVertex2f(wall.right, wall.bottom)
			glVertex2f(wall.left, wall.bottom)
			glEnd()
			
	def drawEndTarget(self):
		glPushMatrix()
		glTranslatef(self.winPos[0], self.winPos[1], 0.0)
		glCallList(self.endTargetDList)
		glPopMatrix()
				
	def getVectorAtPoint(self, point):
		# Fairly sure this works
		result = zeros(2, "f")
	
		x = fmod(point[0], VECTOR_ARROW_SPACING) / VECTOR_ARROW_SPACING
		y = fmod(point[1], VECTOR_ARROW_SPACING) / VECTOR_ARROW_SPACING
	
		point = point / VECTOR_ARROW_SPACING
		# Offset
		point -= 1
		
		result += self.arrowList[int(floor(point[0]))][int(floor(point[1]))].getVec() * (1 - x) * (1 - y)
		if ceil(point[0]) < self.x and ceil(point[0]) >= 0:
			result += self.arrowList[int(ceil(point[0]))][int(floor(point[1]))].getVec() * x * (1 - y)
			if ceil(point[1]) < self.y and ceil(point[1]) >= 0:
				result += self.arrowList[int(ceil(point[0]))][int(ceil(point[1]))].getVec() * x * y
		if ceil(point[1]) < self.y and ceil(point[1]) >= 0:
			result += self.arrowList[int(floor(point[0]))][int(ceil(point[1]))].getVec() * (1 - x) * y
		
		return result
		
	def getWallList(self):
		return self.wallList
		
	def checkVictoryForDisk(self, point):
		if sum((point - self.winPos) ** 2) < DISK_RADIUS_SQ + END_RADIUS_SQ:
			return True
		return False
		
	def checkVictory(self):
		for disk in self.diskList:
			if not disk.isFinished():
				return
				
		self.finishLevel()
				
	def finishLevel(self):	
		self.bFinished = True
		
	def isFinished(self):
		return self.bFinished
		
		
	def rotate(self, bClockWise):
		if bClockWise:
			rot = -ROT_INC
		else:
			rot = ROT_INC
	
		cosRot = cos(rot)
		sinRot = sin(rot)
	
		rotArray = matrix(([cosRot, sinRot], [-sinRot, cosRot]))	
		
		for arrowSubList in self.arrowList:
			for arrow in arrowSubList:
				arrow.setVec(array(arrow.getRotVec() * rotArray)[0])
		
	def update(self):
		for disk in self.diskList:
			if not disk.isFinished():
				disk.update()
			
		self.checkVictory()

	def draw(self):
		self.drawVectorFeild()
		self.drawBorder()
		self.drawWalls()
		self.drawEndTarget()
		
		for disk in self.diskList:
			if not disk.isFinished():
				disk.draw()
			
	def genNextLevel(self):
		if self.iLevel == NUM_LEVELS - 1:
			return None
	
		return World(self.iLevel + 1)
			
